// ArithmeticServer.java

import java.io.*;
import java.net.*;

public class ArithmeticServer {

    // Listening port must be in the unprivileged range [1024-65535!
    // declared as static because
    private static final int PORT = 1948;

    public static void main(String[] args) {

        System.out.println("ArithmeticServer: started");

        DatagramSocket socket = null;
        DatagramPacket packet = null;
        byte[] buf = new byte[256];
        int port = -1;

        // check input arguments: either 0 or 1 argoment (port number)
        if ((args.length == 0)) {
            port = PORT;
        } else if (args.length == 1) {
            try {
                port = Integer.parseInt(args[0]);
                // check that the port is in the unprivileged range
                if (port < 1024 || port > 65535) {
                    System.out.println("Usage: java ArithmeticServer [serverPort>=1024]");
                    System.exit(1);
                }
            } catch (NumberFormatException e) {
                System.out.println("Usage: java ArithmeticServer [serverPort>=1024]");
                System.exit(1);
            }
        } else {
            System.out.println("Usage: java ArithmeticServer [serverPort>=1024]");
            System.exit(1);
        }

        try {
            socket = new DatagramSocket(port);
            packet = new DatagramPacket(buf, buf.length);
            System.out.println("Built socket: " + socket);
        } catch (SocketException e) {
            System.out.println("Error while creating socket: ");
            e.printStackTrace();
            System.exit(1);
        }

        try {

            String operation = null;
            int op1;
            int op2;
            int result = 0;
            ByteArrayInputStream biStream = null;
            DataInputStream diStream = null;
            ByteArrayOutputStream boStream = null;
            DataOutputStream doStream = null;
            byte[] data = null;
            boolean success = true;
            String errorDescription = null;

            while (true) {
                System.out.println("\nWaiting for requests");

                // ricezione del datagramma
                try {
                    success = true;
                    packet.setData(new byte[512]);
                    socket.receive(packet);
                } catch (IOException e) {
                    System.err.println("Error while receiving packet: " + e.getMessage());
                    e.printStackTrace();
                    continue;
                    // server repeats endless loop
                }

                try {
                    biStream = new ByteArrayInputStream(packet.getData(), 0, packet.getLength());
                    diStream = new DataInputStream(biStream);
                    operation = diStream.readUTF();
                    op1 = diStream.readInt();
                    op2 = diStream.readInt();
                    System.out.println("Request: " + op1 + " " + operation + " " + op2);
                    if (operation.equals("+")) {
                        result = op1 + op2;
                    } else if (operation.equals("-")) {
                        result = op1 - op2;
                    } else if (operation.equals("*")) {
                        result = op1 * op2;
                    } else if (operation.equals("/")) {
                        if (op2 != 0) {
                            result = op1 / op2;
                        } else {
                            success = false;
                            errorDescription = "Can not divide by zero!";
                        }
                    } else {
                        success = false;
                        errorDescription = "Unknown operation \"" + operation + "\"";
                    }
                } catch (Exception e) {
                    success = false;
                    errorDescription = e.getMessage();
                }

                // build response and send it
                try {
                    boStream = new ByteArrayOutputStream();
                    doStream = new DataOutputStream(boStream);
                    if (success) {
                        doStream.writeUTF("success");
                        doStream.writeInt(result);
                    } else {
                        doStream.writeUTF("failure");
                        doStream.writeUTF(errorDescription);
                    }
                    doStream.flush();
                    data = boStream.toByteArray();
                    packet.setData(data, 0, data.length);
                    socket.send(packet);
                } catch (IOException e) {
                    System.err.println("Error while sending response: " + e.getMessage());
                    e.printStackTrace();
                    continue;
                    // server repeats endless loop
                }

            } // while

        } // catch all exceptions not managed in the main loop and terminate
        catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println("ArithmeticServer: terminating...");
        socket.close();
    }
}
